'=============================================================================
' Copyright  Micromega Corporation 2002-2004. All rights reserved
'
' File: demo.bas
'
' Demonstration routine for uM-FPU V1.0 using meLabs PICBASIC Pro compiler.
'
' @author Cam Thompson, Micromega Corporation
' @version V1.0 - April 29, 2004
'   V1.1 - January 14, 2005
'     - modified to support meLabs PICBASIC Pro compiler
'   V1.0 - April 29, 2004
'     - original version
'=============================================================================

    @ERRORLEVEL -306                    'turn off page bit warning

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      'specify the speed of the oscillator
OSC_SPEED   con 20                      'define PICBASIC constant

'-------------------- debug definitions ---------------------------------------
' The routines in umfpuV1-PBP.bas use the DEBUG command to print floating point
' and long integer numbers. The serial pin and baud rate for the DEBUG command
' are specified with the following DEFINES. These should be changed to suit
' your application.

define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  9600
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
' The following assignments define the pins used to connect to the uM-FPU V1.0
' chip. These should be changed to suit your application.

FpuClock    var         PORTC.3
FpuDataIn   var         PORTC.4
FpuDataOut  var         PORTC.4

include     "umfpuV1.bas"               'include the uM-FPU support routines

'-------------------- uM-FPU register definitions ----------------------------

Result          CON     1               'result
Angle           CON     2               'angle
Multiplier      CON     3               'multiplier
DiameterIn      CON     4               'diameter in inches
Circumference   CON     5               'circumference
Area            CON     6               'area
Pi              CON     7               'constant pi
F2_0            CON     8               'constant 2.0
F2_54           CON     9               'constant 2.54

'-------------------- variables ----------------------------------------------

diameterCm      VAR     word            'diameter in centimeters

'-------------------- EEPROM data --------------------------------------------

Pi_Str      DATA  "3.1415927", 0        'zero terminated string
F20_0Str    DATA  "20.0", 0             'zero terminated string
F20_0       DATA  $41, $A0, $00, $00    '20.0
F0_5        DATA  $3F, $00, $00, $00    '0.5

'-------------------- initialization -----------------------------------------

reset:
  DEBUG 13, 10, 13, 10, "uM-FPU Demo Program", 13, 10

  'An FPU_Reset call must be done at the start of the program to establish
  'proper communications with the uM-FPU. The following example displays a
  'message if the reset fails, or the uM-FPU version string if successful.

  GOSUB FPU_Reset                       'reset the FPU hardware
  IF fStatus = 0 THEN
    DEBUG "uM-FPU not detected."
    END
      ELSE
    GOSUB FPU_Version                   'display the uM-FPU version number
    DEBUG 13, 10
  ENDIF

'-------------------- main routine -------------------------------------------
'The following programs demonstrates how to use the various uM-FPU support
'routines.  Each operation is explained.

  DEBUG 13, 10, "Read comments in program for further desciption.", 13, 10

  'There are sixteen 32-bit registers on the uM-FPU numbered 0 through 15.
  'Register 0 is used as a working register by the uM-FPU and should not
  'generally be used by the application code. Registers 1 through 15 are
  'available for general use. The user selects the registers to be involved
  'in an arithmetic operation by setting the value of fA and fB to select the
  'A and B registers.

  'There are various ways to load data into the uM-FPU registers.
  'Several examples are shown.

  'To set A to the value of a local 8-bit integer.
  '-----------------------------------------------
  'The Load_FloatInt routine loads the 8-bit integer stored in fLow.LOWBYTE to
  'uM-FPU register 0, converts it to floating point, and sets fB = 0.

  DEBUG "set A (Load_FloatByte): "
  fA = Angle                            'set Angle to integer value 20
  fLow.LOWBYTE = 20
  GOSUB Load_FloatByte
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'To set A to the value of a local floating point number.
  '-------------------------------------------------------
  'The Load_Float routine loads the 32-bit value stored in fHigh and fLow to
  'uM-FPU register 0, and sets fB = 0.

  DEBUG "set A (Load_Float): "
  fA = Angle                            'set Angle to float value 0.75
  fHigh = $3F40                         'high 16 bits of constant 0.75
  fLow = $0000                          'low 16 bits of constant 0.75
  GOSUB Load_Float
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'To set A to the value of a floating point constant stored in EPROM.
  '-------------------------------------------------------------------
  'The Load_FloatData routine loads the 32-bit value stored in EEPROM to
  'uM-FPU register 0 and sets fB = 0.

  DEBUG "set A (Load_FloatData): "
  fA = Angle                            'set Angle to float constant 20.0
  fAddr = F20_0                         'get EEPROM address
  GOSUB Load_FloatData
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'To set A to the value of an ASCII string stored in EEPROM.
  '----------------------------------------------------------
  'The Load_FloatStr routine sends the ASCII string stored in the EEPROM to
  'the uM-FPU.  The string is converted to a 32-bit floating point number
  'and stored in register 0, and fB is set to 0.

  DEBUG "set A (Load_FloatStr): "
  fA = Angle                            'set Angle to value of string
  fAddr = Pi_Str                        'get EEPROM address
  GOSUB Load_FloatStr
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'To set A to the value of register B.
  '------------------------------------
  DEBUG "set A = B: "
  fA = Result
  fB = Angle
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'The basic arithmetic operators are supported by the following routines.
  '   GOSUB Fadd       'A = A + B
  '   GOSUB Fsubract   'A = A - B
  '   GOSUB Fmultiply  'A = A * B
  '   GOSUB Fdivide    'A = A / B

  'An example of a basic arithmetic operation
  '------------------------------------------
  DEBUG "Multiplier: "
  fA = Multiplier                       'load multiplier with 0.5
  fAddr = F0_5
  GOSUB Load_FloatData
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  DEBUG "Result: "
  fA = Result
  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  DEBUG "Result = Result * Multiplier: "
  fA = Result                           'result = result * multiplier
  fB = Multiplier
  GOSUB Fmultiply

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'An example of an arithmetic function
  '------------------------------------
  DEBUG "Angle: "
  fA = Angle                            'angle = pi
  fAddr = Pi_Str
  GOSUB Load_FloatStr
  GOSUB Load_Pi
  GOSUB Fset

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  DEBUG "cos(Angle): "
  fA = Angle                            'calculate the cosine of angle
  GOSUB Fcos

  GOSUB Print_Float                     'print the result
  DEBUG 13, 10

  'Use the FgetStatus routine to check the status of a uM-FPU register value.
  'The status of the selected register is loaded into the fStatus local variable
  'and can be checked using predefined bit variables. The value of fStatus is only
  'valid until the next uM-FPU operation occurs.  The status is checked as follows:
  ' IF (fStatus_Zero = 1)     'value is zero
  ' IF (fStatus_Sign = 1)     'value negative
  ' IF (fStatus_NaN = 1)      'value is Not a Number
  ' IF (fStatus_Inf = 1)      'value is infinite

  'Check if number is negative
  '---------------------------
  fA = Angle                            'check for negative cosine
  GOSUB FgetStatus
  IF fStatus_Sign = 1 THEN DEBUG "cos(Angle) is negative", 13, 10

  'Use the Fcompare routine to compare two floating point values. The fStatus
  'value will be set for the value of (A - B).
  ' IF (fStatus_Zero = 1)     'A = B
  ' ELSEIF (fStatus_Sign = 1) 'A < B
  ' ELSE                      'A > B

  'Compare two values
  '-------------------
  DEBUG "Angle: "
  fA = Angle
  GOSUB Print_Float
  DEBUG 13, 10

  DEBUG "Result: "
  fA = Result
  GOSUB Print_Float
  DEBUG 13, 10

  fA = Angle
  fB = Result
  GOSUB Fcompare

  IF (fStatus_Zero = 1) THEN
    DEBUG "Angle = Result"
  ELSE
   IF (fStatus_Sign = 1) THEN
     DEBUG "Angle < Result"
   ELSE
     DEBUG "Angle > Result"
   ENDIF
  ENDIF

  DEBUG 13, 10, "Done.", 13, 10         'end of program
  END
