'==============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' File: umfpuV1.BAS
'
' This file provide support for using the uM-FPU V1.0 floating point coprocessor
' with the meLabs PIC Basic Pro compiler. This file is added to the main program
' using an include statement.  The user must set the definitions for the
' oscillator speed and interface pins appropriately. Unused routines can be
' deleted to save space, but it is recommended that all uM-FPU definitions are
' retained. Internal support routines have a 'Sys_' prefix (e.g. Sys_Command).
' These routines are called by other functions in the library and will need to
' be included depending on which functions are used. Most of the system
' functions at the end of this file will be required. In order to reduce memory
' requirements, some routines fall through to the code that follows it.  All of
' these cases are clearly commented. When deleting unused routines be sure to
' check if any following code is required.
'
' @author Cam Thompson, Micromega Corporation
' @version V1.1 - January 10, 2005
' changes:
'   V1.1 - January 10, 2005
'     - modified for meLabs PicBASIC Pro compatibility by Steve Roberts
'     - modifications to support higher clock speeds by Cam Thompson
'   V1.0 - April 29, 2004
'     - original version
'==============================================================================

'-------------------- uM-FPU V1.0 definitions ---------------------------------

'jump to end of code
'(in case this file is included at beginning of the program)
  goto FPU_end

'-------------------- PIC oscillator speed ------------------------------------
' Note: The oscillator speed must be defined to ensure proper timing constants
' for the uM-FPU routines. These definitions are normally included in the main
' program. An example of the required definitions is as follows:
'
'   define      OSC 20                  'specify the speed of the oscillator
'   OSC_SPEED   con 20                  'define PICBASIC constant

'-------------------- uM-FPU pin definitions ----------------------------------
' Note: FpuClock and FpuData must be defined according to the pin assignments
' used in your application. These definitions are normally made in the main
' program. If a 2-wire SPI interface is used, the definition for FpuDataIn and
' FpuDataOut would be the same. An example of the definitions for a 2-wire SPI
' interface is as follows:
'
'    FpuClock   var      PORTC.3
'    FpuDataIn  var      PORTC.4
'    FpuDataOut var      PORTC.4
'
' An example of the definitions for a 3-wire SPI interface is as follows:
'    FpuClock   var      PORTC.3
'    FpuDataIn  var      PORTC.4
'    FpuDataOut var      PORTC.5

'-------------------- uM-FPU constants ----------------------------------------

DataPeriod      con      25                             '25 usec data period
BitTime         con      2 + 80 / OSC_SPEED             'time to transmit a bit
WriteDelay      con      DataPeriod - 100 / OSC_SPEED   'delay after write opcode
ReadDelay       con      DataPeriod - BitTime           'delay before first read byte

SELECTB_OP      con      $10            'SELECTB opcode
WRITEB_OP       con      $30            'WRITEB opcode

MSBFIRST        con      1              'shiftout mode
MSBPRE          con      0              'shiftin mode

'-------------------- uM-FPU variables ----------------------------------------

fHigh           var     word            'high 16-bits of float or long
fLow            var     word            'low 16-bits of float of long
fAddr           var     fLow            'EEPROM address (can be used by other routines)
fFormat         var     fLow.LOWBYTE    'format byte for formatted printing
fA              var     byte            'A register
fB              var     byte            'B register
fOpcode         var     byte            'opcode
fStatus         var     fOpcode         'status from last FStatus call
fStatus_Zero    var     fStatus.0       'Zero status bit (0-not zero, 1-zero)
fStatus_Sign    var     fstatus.1       'Sign status bit (0-positive, 1-negative)
fStatus_NaN     var     fstatus.2       'Not a Number status bit (0-valid number, 1-NaN)
fStatus_Inf     var     fStatus.3       'Infinity status bit (0-not infinite, 1-infinite)
fChar           var     fOpcode         'temporary character variable
fLastA          var     byte            'last A register selected

'-------------------- Initialization and Setup Routines -----------------------

FPU_Reset:
  LOW FpuClock                          'set clock and data lines low
  LOW FpuDataOut

  FpuClock = 1                          'send reset pulse to uM-FPU
  PAUSEUS 300
  FpuClock = 0
  PAUSE 8

  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [$F0]
  PAUSEUS DataPeriod*2
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fChar]

  IF (fChar = $5C) THEN                 'confirm communications
    fStatus = 1                         'sync test successful
  ELSE
    fStatus = 0                         'sync test failed
  ENDIF

  fA = 0                                'initialize the uM-FPU variables
  fB = 0
  fLastA = 0
  RETURN

FPU_Preload:                            'load all preload values
  READ fAddr, fChar                     'get the register value
  fAddr = fAddr + 1                     'check for terminator
  IF (fChar = 0) OR (fchar > 15) THEN RETURN 'yes, then exit
  fA = fChar & $0F                      'select the register
  GOSUB Sys_FloatData                   'load value into register
  GOSUB Fset
  goto FPU_Preload

FPU_Version:
  fOpcode = $FF                         'print the version number and exit
  GOTO Sys_PrintStr

'-------------------- Left, Right Parenthesis ---------------------------------

Left:
  fOpcode = $EE                         'left parenthesis
  GOTO Sys_Xcommand

Right:
  fB = 0
  fOpcode = $EF                         'right parenthesis
  GOTO Sys_Xcommand

'-------------------- Floating Point Routines ---------------------------------

Fabs:
  fOpcode = $EC                         'A = |A|
  GOTO Sys_Command
Facos:
  fOpcode = $E6                         'A = acos(A) radians
  GOTO Sys_Xcommand
Fadd:
  fOpcode = $60                         'A = A + B
  GOTO Sys_Operator
Fasin:
  fOpcode = $E5                         'A = asin(A) radians
  GOTO Sys_Xcommand
Fatan:
  fOpcode = $E7                         'A = atan(A) radians
  GOTO Sys_Xcommand
Fatan2:
  fOpcode = $E8                         'A = atan(A/B)
  GOTO Sys_CommandB
Fceil:
  fOpcode = $E9                         'A = nearest integer >= A
  GOTO Sys_Command
Fcompare:
  fOpcode = $F3                         'compare registers A and B (A - B)
  GOSUB Sys_CommandB
  GOTO Sys_ReadStatus
Fcos:
  fOpcode = $E6                         'A = cos(A) radians
  GOTO Sys_Command
Fdivide:
  fOpcode = $90                         'A = A / B
  GOTO Sys_Operator
Fexp:
  fOpcode = $E3                         'A = exp(A)
  GOTO Sys_Command
Fexp10:
  fOpcode = $E4                         'A = exp10(A)
  GOTO Sys_Command
Ffix:
  fOpcode = $F2                         'A = fix(A), (convert float to long)
  GOSUB Sys_Command
  fB = 0
  fOpCode = $50
  GOTO Sys_Command
Ffloor:
  fOpcode = $E8                         'A = nearest integer <= A
  GOTO Sys_Command
Lget:
Fget:
  fOpcode = $40 + fA                   'get value of A
  GOSUB Sys_Command
  IF OSC_SPEED > 4 THEN PAUSEUS ReadDelay
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fHigh.HIGHBYTE]
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fHigh.LOWBYTE]
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fLow.HIGHBYTE]
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fLow.LOWBYTE]
  RETURN
FgetStatus:
  fOpcode = $FD                         'get floating point status of A
  GOSUB Sys_Command
  GOTO Sys_ReadStatus
Finverse:
  fOpcode = $ED                         'A = 1 / A
  GOTO Sys_Command
Flog:
  fOpcode = $E1                         'A = log(A)
  GOTO Sys_Command
Flog10:
  fOpcode = $E2                         'A = log10(A)
  GOTO Sys_Command
Fmax:
  fOpcode = $E3                         'A = maximum of A AND B
  GOTO Sys_CommandB
Fmin:
  fOpCode = $E2                         'A = minimum of A AND B
  GOTO Sys_CommandB
Fmultiply:
  fOpcode = $80                         'A = A * B
  GOTO Sys_Operator
Fnegate:
  fOpcode = $EB                         'A = -A
  GOTO Sys_Command
Fpower:
  fOpcode = $E0                         'A = A to the power of B
  GOTO Sys_CommandB
Froot:
  fOpcode = $E1                         'A = the Bth root of A
  GOTO Sys_CommandB
Fround:
  fOpcode = $EA                         'A = nearest integer to A
  GOTO Sys_Command
Lset:
Fset:
  fOpcode = $50                         'A = B
  GOTO Sys_Operator
Fsin:
  fOpcode = $E5                         'A = sin(A) radians
  GOTO Sys_Command
Fsqrt:
  fOpcode = $E0                         'A = sqrt(A)
  GOTO Sys_Command
Fsubtract:
  fOpcode = $70                         'A = A - B
  GOTO Sys_Operator
Ftan:
  fOpcode = $E7                         'A = tan(A) radians
  GOTO Sys_Command
FtoDegrees:
  fOpcode = $EE                         'A = A / (PI / 180.0) radians to degrees
  GOTO Sys_Command
FtoRadians:
  fOpcode = $EF                         'A = A * (PI / 180.0) degrees to radians
  GOTO Sys_Command

'-------------------- Long Integer Routines -----------------------------------

Labs:
  fOpcode = $ED                         'A = |A| (long integer)
  GOTO Sys_Xcommand
Ladd:
  fOpcode = $A0                         'A = A + B (long integer)
  GOTO Sys_Operator
Lcompare:
  fOpcode = $E9                         'compare A and B (signed long integer)
  GOSUB Sys_CommandB
  GOTO Sys_ReadStatus
Ldivide:
  fOpcode = $D0                         'A = A / B (signed long integer)
  GOTO Sys_Operator
Lfloat:
  fOpcode = $F1                         'A = float(A), (convert long to float)
  GOSUB Sys_Command
  fB = 0
  fOpCode = $50
  GOTO Sys_Command
LgetStatus:
  fOpcode = $EB                         'get status of A (long integer)
  GOSUB Sys_Xcommand
  GOTO Sys_ReadStatus
Lmultiply:
  fOpcode = $C0                         'A = A * B (long integer)
  GOTO Sys_Operator
Lnegate:
  fOpcode = $EC                         'A = -A (long integer)
  GOTO Sys_Xcommand
Lsubtract:
  fOpcode = $B0                         'A = A - B (long integer)
  GOTO Sys_Operator
Lucompare:
  fOpcode = $EA                         'compare A and B (unsigned long integer)
  GOSUB Sys_CommandB
  GOTO Sys_ReadStatus
Ludivide:
  fOpcode = $D0 + fB                    'A = A / B (unsigned long integer)
  GOTO Sys_Xcommand

'-------------------- Load Routines -------------------------------------------

Load_E:
  fOpcode = $F2                         'load e (2.7182817)
  GOTO Sys_LoadConstant
Load_Long:
Load_Float:
Sys_Float:
  IF (FpuDataIn = 1) THEN Sys_Float     'wait until uM-FPU is ready
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [WRITEB_OP, fHigh\16, fLow\16]
  INPUT FpuDataIn                  'set the data pin for input
  fB = 0
  RETURN
Load_FloatByte:
  fOpcode = $F4                         'load signed byte, float
                                        '(fall through to Sys_FloatByte)
Sys_FloatByte:
  GOSUB Sys_Command                     'send opcode and data byte
                                        '(fall through to Sys_FloatByte2)
Sys_FloatByte2:
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fLow]
  INPUT FpuDataIn                       'set the data pin for input
  fB = 0
  RETURN

Load_FloatData:
Sys_FloatData:

  IF (FpuDataIn = 1) THEN Sys_FloatData 'wait until uM-FPU is ready
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [WRITEB_OP]

Sys_SendData:
  FOR fB = 1 TO 4                       'send 4 byes of data from EPROM
   READ fAddr, fChar
   SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fChar]
   fAddr = fAddr + 1
  NEXT
  INPUT FpuDataIn                       'set the data pin for input
  fB = 0
  RETURN

Load_FloatStr:
  fOpcode = $F9                         'convert ASCII to float
                                        '(fall through to Sys_LoadStr)
Sys_LoadStr:
  GOSUB Sys_Command                     'send string command
                                        '(fall through to Sys_LoadStr2)
Sys_LoadStr2:
  READ fAddr, fChar
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fChar]
  IF (fChar = 0) THEN Sys_Loadstr3      'check for zero terminator
  IF OSC_SPEED > 4 THEN PAUSEUS DataPeriod*2
  fAddr = fAddr + 1
  GOTO Sys_LoadStr2

Sys_Loadstr3:
  INPUT FpuDataIn                       'set the data pin for input
  fB = 0
  RETURN

Load_FloatUByte:
  fOpcode = $F5                         'load unsigned byte, float
  GOTO Sys_FloatByte
Load_FloatUWord:
  fOpcode = $F7                         'load unsigned word
  GOTO Sys_FloatWord
Load_FloatWord:
  fOpcode = $F6                         'load signed word
                                        '(fall through to Sys_FloatWord)
Sys_FloatWord:
  GOSUB Sys_Command                     'send load opcode and two data bytes
                                        '(fall through to Sys_FloatWord2)
Sys_FloatWord2:
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fLow\16]
  INPUT FpuDataIn                       'set the data pin for input
  fB = 0
  RETURN

Load_Fraction:
  fOpcode = $E4                         'load register 0 with the fractional part of A
  fB = 0
  GOTO Sys_Xcommand
Load_LongByte:
  fOpcode = $F4                         'load signed byte, long
                                        '(fall through to Sys_LongByte)
Sys_LongByte:
  GOSUB Sys_Xcommand                    'send opcode and data byte
  GOTO Sys_FloatByte2

Load_LongData:
  GOTO Sys_FloatData
Load_LongStr:
  fOpcode = $FB                         'convert ASCII to long integer
  GOTO Sys_LoadStr
Load_LongUByte:
  fOpcode = $F5                         'load unsigned byte, long
  GOTO Sys_LongByte
Load_LongUWord:
  fOpcode = $F7                         'load unsigned word
  GOTO Sys_LongWord
Load_LongWord:
  fOpcode = $F6                         'load signed word
                                        '(fall through to Sys_LongWord)
Sys_LongWord:
  GOSUB Sys_Xcommand                    'send opcode and two data bytes
  GOTO Sys_FloatWord2

Load_One:
  fOpcode = $F1                         'load 1.0
  GOTO Sys_LoadConstant
Load_Pi:
  fOpcode = $F3                         'load pi (3.1415927)
  GOTO Sys_LoadConstant
Load_Zero:
  fOpcode = $F0                         'load 0.0
                                        '(fall through to Sys_LoadConstant)
Sys_LoadConstant:
  fB = 0
  GOTO Sys_Xcommand

'-------------------- Print Routines ------------------------------------------

Print_Float:
  fLow = 0                              'set format to zero (free format)
                                        '(fall through to Print_FloatFormat)
Print_FloatFormat:
  flow.highbyte = fA                    'workaround to correct V1.0 bug
  fA = 0                                'This fix ensured that the minus sign
  GOSUB Fabs                            ' is displayed correctly by Print_Format.
  GOSUB Fabs
  fA = fLow.highbyte

  fOpcode = $FA                         'convert floating point to formatted ASCII
  GOTO Sys_PrintFormat

Print_Hex:
  GOSUB Fget                            'get the 32-bit value
  DEBUG IHEX4 fHigh, HEX4 fLow          'print the hex value
  RETURN

Print_Long:
  fLow = 0                              'set format to 0 (free format)
                                        '(fall through to Print_LongFormat)
Print_LongFormat:
  fOpcode = $FC                         'convert long integer to formatted ASCII
                                        '(fall through to Sys_PrintFormat)
Sys_PrintFormat:
  GOSUB Sys_Command                     'send conversion command
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fLow] 'send format code
  INPUT FpuDataIn                       'set the data pin for input
  GOTO Sys_PrintStr2                    'display string and exit

Sys_PrintStr:
  GOSUB Sys_Command                     'send string command
                                        '(fall through to Sys_PrintStr2)
Sys_PrintStr2:
  fOpcode = $F8                         'send read string opcode
  GOSUB Sys_Command
  IF OSC_SPEED > 4 THEN PAUSEUS ReadDelay

Sys_PrintStr3:                          'display zero terminated string
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fChar]
  IF (fChar = 0 OR fChar > 127) THEN RETURN
  DEBUG fChar
  GOTO Sys_PrintStr3

'-------------------- System Routines -----------------------------------------

Sys_ReadStatus:
  IF OSC_SPEED > 4 THEN PAUSEUS ReadDelay
  SHIFTIN FpuDataIn, FpuClock, MSBPRE, [fStatus] 'get status bits
  RETURN                                'exit


Sys_CommandB:
  IF (FpuDataIn = 1) THEN Sys_CommandB   'wait until uM-FPU is ready
  SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [SELECTB_OP+fB] 'select B register
  INPUT FpuDataIn                       'set the data pin for input
  IF (fOpcode = $F3) THEN Sys_Command
                              '(fall through to Sys_Xcommand)
Sys_Xcommand:
  fHigh.LOWBYTE = fOpcode               'save original opcode
  fOpcode = $FE                         'send xop prefix
  GOSUB Sys_Command
  fOpCode = fHigh.LOWBYTE               'send original opcode
  GOTO Sys_Command2

Sys_Operator:
  fOpcode = fOpcode + fB                'set B register
                                        '(fall through to Sys_Command)
Sys_Command:
  IF (FpuDataIn = 1) THEN Sys_Command   'wait until uM-FPU is ready
                                        '(fall through to Sys_Command2)
Sys_Command2:
  IF (fA <> fLastA) THEN                'if required, select A register
   SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fA, fOpcode]
   fLastA = fA
  ELSE                                  'send opcode
   SHIFTOUT FpuDataOut, FpuClock, MSBFIRST, [fOpcode]
  ENDIF
  INPUT FpuDataIn                       'set the data pin for input
  IF OSC_SPEED > 4 THEN                 'ensure minimum data period
   PAUSEUS WriteDelay
  ENDIF

  RETURN

FPU_end:                                'end of umfpuV1_PBP.bas
